/* 
*
*  ProLinga-Data
*
*  Copyright (C) 2002-2008 Xobas Software.
*  All rights reserved.
*  
*  This file is part of ProLinga-Data.
*
*  ProLinga-Data is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or
*  (at your option) any later version.
*
*  ProLinga-Data is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with ProLinga-Data.  If not, see <http://www.gnu.org/licenses/>.
*
*  More information is available at the following addresses:
*
*  Website     : http://www.prolinga.org
*
*  Email       : prolinga-list@prolinga.org
*
*
*/

#ifndef __DAT_CONFIG_HPP
#define __DAT_CONFIG_HPP

#define PRODUCT_NAME		"Data"

/* Default configuration settings */
#define DEF_CONFIG_FILE		SYSCONFDIR"/prolingadatcfg.xml"	/* Config file name */
#define DEF_LOG_FILE		"prolinga.log"		/* Log file name */
#define DEF_LOG_LEVEL		2			/* Log Level */
#define DEF_CLIENT_HOST		"localhost"		/* Client Hostname or IP */
#define DEF_CLIENT_PORT		8007			/* Client TCP port */
#define DEF_CLIENT_COMP_LEVEL	6			/* Compression Level (range 0-9) */
#define DEF_SERVER_HOST		"localhost"		/* Server Hostname or IP */
#define DEF_SERVER_PORT		8007			/* Server TCP port */
#define DEF_SERVER_COMP_LEVEL	6			/* Compression Level (range 0-9) */
#define DEF_DATA_PATH		"./"			/* Data Path */
#define DEF_THREADS		5			/* Number of Threads */	

/* XML Config keys */
#define ENV_KEY_LOGFILE		"LogFile"
#define ENV_KEY_LOGLEVEL	"LogLevel"
#define ENV_KEY_DATA_PATH	"DataPath"
#define ENV_KEY_THREADS		"NumberOfThreads"
#define ENV_KEY_CLIENT_HOST	"ClientHost"
#define ENV_KEY_CLIENT_PORT	"ClientPort"
#define ENV_KEY_CLIENT_COMP	"ClientCompressionLevel"
#define ENV_KEY_SERVER_HOST	"ServerHost"
#define ENV_KEY_SERVER_PORT	"ServerPort"
#define ENV_KEY_SERVER_COMP	"ServerCompressionLevel"

/* Log Levels */
#define LOG_LEVEL_0	0	/* No logging */
#define LOG_LEVEL_1	1	/* Free */      
#define LOG_LEVEL_2	2	/* + Messages/Errors */
#define LOG_LEVEL_3	3	/* Free */
#define LOG_LEVEL_4	4	/* + Soap request/response info */
#define LOG_LEVEL_5	5	/* Free */

class ConfigDatSetting
{
	friend class ConfigDat;
                                                                                                                                             
	public:
		ConfigDatSetting();
		~ConfigDatSetting();
	private:
		char *getStringValue(ConfigDatSetting *confsetPtr, const char *name, const char *defValue);
		int getNumberValue(ConfigDatSetting *confsetPtr, const char *name, const int defValue);
		void putStringValue(ConfigDatSetting **confsetPtr, const char *name, const char *value);
		void putNumberValue(ConfigDatSetting **confsetPtr, const char *name, const int value);
		void deleteList(ConfigDatSetting **confsetPtr);
		void printList(ConfigDatSetting *confsetPtr) const;
		char confsetName[32];
		char *confsetStringValue;
		int confsetNumberValue;
		ConfigDatSetting *nextPtr;
};
typedef ConfigDatSetting *ConfigDatSettingPtr;

class ConfigDat
{
	public:
		ConfigDat(const char *configFileName);
		~ConfigDat();
		char *getStringValue(const char *name, const char *defValue);
		int getNumberValue(const char *name, const int defValue);
		void putStringValue(const char *name, const char *value);
		void putNumberValue(const char *name, const int value);
		void writeConfig(const char *logFileName) const;
		void printList(void) const;
	private:
		ConfigDatSetting *startPtr;
};
typedef ConfigDat *ConfigDatPtr;

#endif /* __DAT_CONFIG_HPP */

